/**
 * Bind a function or object deeply
 * @module bind-deep
 * @author Evelyn Hathaway
 * @license MIT
 */
"use strict";

var bindDeep = function bindDeep(object, thisArg) {
  if (!(object instanceof Object)) {
    return object;
  }

  var bound;
  var prototype = Object.getPrototypeOf(object);

  for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  if (object instanceof Function) {
    if (typeof object.bind === "function") {
      bound = object.bind.apply(object, [thisArg].concat(args));
    }
  } else if (Array.isArray(object)) {
    bound = [];

    for (var i = 0; i < object.length; i++) {
      bound[i] = bindDeep.apply(void 0, [object[i], thisArg].concat(args));
    }

    if (prototype !== Array.prototype) {
      Object.setPrototypeOf(bound, prototype);
    }
  }

  if (typeof bound === "undefined") {
    bound = Object.create(prototype);
  }

  var _arr = Object.keys(object);

  for (var _i = 0; _i < _arr.length; _i++) {
    var key = _arr[_i];

    if (Object.hasOwnProperty(bound, key)) {
      continue;
    }

    var descriptor = Object.getOwnPropertyDescriptor(object, key);
    var _arr2 = ["value", "set", "get"];

    for (var _i2 = 0; _i2 < _arr2.length; _i2++) {
      var _key2 = _arr2[_i2];

      if (descriptor[_key2]) {
        descriptor[_key2] = bindDeep.apply(void 0, [descriptor[_key2], thisArg].concat(args));
      }
    }

    Object.defineProperty(bound, key, descriptor);
  }

  return bound;
};

module.exports = bindDeep;